@@ -202,6 +202,33 @@ void print_cap(struct video_capability *vidcap, struct video_window *vidwin,
 	}
 }
 /* }}} */
+
+/* {{{ [fold] read_settings() */
+void read_settings(struct video_picture *vidpic) {
+	if (vidpic) {
+		printf("Brightness      : %i\n", vidpic->brightness);
+		printf("Hue             : %i\n", vidpic->hue);
+		printf("Color           : %i\n", vidpic->colour);
+		printf("Contrast        : %i\n", vidpic->contrast);
+		printf("Whiteness       : %i\n", vidpic->whiteness);
+		printf("Depth           : %i\n", vidpic->depth);
+	}
+}
+/* }}} */
+/* {{{ [fold] write_settings() */
+void write_settings(struct video_picture *vidpic) {
+	int val, r;
+	if (vidpic) {
+		r = scanf("Brightness       : %i\n", &val); if (r>0) vidpic->brightness = val;
+		r = scanf("Hue              : %i\n", &val); if (r>0) vidpic->hue = val;
+		r = scanf("Color            : %i\n", &val); if (r>0) vidpic->colour = val;
+		r = scanf("Contrast         : %i\n", &val); if (r>0) vidpic->contrast = val;
+		r = scanf("Whiteness        : %i\n", &val); if (r>0) vidpic->whiteness = val;
+		r = scanf("Depth            : %i\n", &val); if (r>0) vidpic->depth = val;
+	}
+}
+/* }}} */
+
 /* {{{ [fold] error() */
 void error(const char *format, ...) {
 	va_list ap;
@@ -234,6 +261,8 @@ void help(void) {
 "	-o val	Set color strength (increase when setting hue)\n"
 "	-c val	Set contrast (camera gain)\n"
 "	-w val	Set whiteness (image sharpness when quality=best)\n"
+"	-s r	Settings read (from STDIN)\n"
+"	-s w	Settings write (to STDOUT)\n"
 "	-g v	Enable gamma correction with the given gamma value\n"
 "	-g rg:gg:bg\n"
 "		Specify different gamma value for each color channel\n"
@@ -541,6 +570,26 @@ int main(int argc, char *argv[])
 					if (r!=0) error("ioctl VIDIOCSPICT");
 					argv += c;
 					break;
+				case 's':	/* Settings read/write */
+					if (!argp) error("missing parameter to option `-%c'", argv[0][1]);
+					switch (argp[0]) {
+					case 'r':	/* read */
+						r = ioctl(fd, VIDIOCGPICT, &vidpic);
+						if (r!=0) error("ioctl VIDIOCGPICT");
+						read_settings(&vidpic);
+						break;
+					case 'w':	/* write */
+						r = ioctl(fd, VIDIOCGPICT, &vidpic);
+						if (r!=0) error("ioctl VIDIOCGPICT");
+						write_settings(&vidpic);		/* Modify only settings which user gives */
+						r = ioctl(fd, VIDIOCSPICT, &vidpic);
+						if (r!=0) error("ioctl VIDIOCSPICT");
+						break;
+					default:	/* unknown */
+						error("unknown request");
+					}
+					argv += c;
+					break;
 				case 'g':
 				case 'e': {
 					struct qc_userlut userlut;
