@@ -7,7 +7,7 @@
 #
 # quickcam.sh - Automated driver build and installation system
 #
-# Copyright (C) 2003  Tuukka Toivonen
+# Copyright (C) 2003-2004  Tuukka Toivonen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -27,13 +27,17 @@
 # }}}
 
 # {{{ [fold] Helper functions
+KERNEL26X=132608
+
 askreturn() {
 	echo -n "Press Ctrl+C to quit, Enter to continue ---> "
 	read x
+	echo ""
 }
 
 askreturnfail() {
-	echo "WARNING: I'll try to continue anyway, but this probably will fail."
+	echo "WARNING: If you press Enter, I'll try to continue anyway,"
+	echo "but this probably will fail. You SHOULD press Ctrl+C now."
 	askreturn
 }
 
@@ -64,7 +68,7 @@ findprog() {
 qcrealpath() {
 	P="$1"
 	while [ -L "$P" ]; do
-		P="`readlink "$P"`"
+		P="`readlink -f -n "$P"`"
 	done
 	if [ ! -e "$P" ]; then
 		return 1
@@ -72,6 +76,23 @@ qcrealpath() {
 	echo "$P"
 	return 0
 }
+
+configurekernel() {
+	if [ "$KERN_PATCHLEVEL" -ge "6" ]; then
+		(cd "$LINUX_DIR" && make oldconfig && make modules_prepare)
+	elif [ "$KERN_PATCHLEVEL" -le "4" ]; then
+		(cd "$LINUX_DIR" && make oldconfig && make dep)
+	else
+		echo "[!] Unknown kernel patchlevel $KERN_PATCHLEVEL"
+		return 1
+	fi
+	if [ "$?" != "0" ]; then
+		echo "[!] Kernel configuration failed, see messages above."
+		askreturnfail
+		return 1
+	fi
+	return 0
+}
 # }}}
 # {{{ [fold] Special root code
 if  [ "$QCINSTCMD" != "" ]; then
@@ -82,7 +103,11 @@ if  [ "$QCINSTCMD" != "" ]; then
 	fi
 	echo "=== Entering root mode ==="
 	R=0
-	if [ "$QCINSTCMD" = "mod" ]; then
+	if [ "$QCINSTCMD" = "conf" ]; then
+		echo "Trying to configure kernel source as root..."
+		configurekernel
+		R="$?"
+	elif [ "$QCINSTCMD" = "mod" ]; then
 		# Load necessary modules
 		echo "Now you will see some error messages."
 		echo "They are probably harmless and you can ignore them"
@@ -104,14 +129,14 @@ if  [ "$QCINSTCMD" != "" ]; then
 			echo "	Alt + SysRq + B: Reboot system immediately"
 			askreturn
 			echo "1" >/proc/sys/kernel/sysrq
-			dmesg -c >/dev/null
-			echo "Now I finally will try to load the module."
-			echo "If you're unlucky, your computer might crash right now!!!!"
-			echo "Consider long if you really want to continue."
-			askreturn
-			echo "You decided to do it, here we go..."
-			$INSMOD "./$MODULE_NAME"
 		fi
+		echo "Now I finally will try to load the module."
+		echo "If you're unlucky, your computer might crash right now!!!!"
+		echo "Consider long if you really want to continue."
+		askreturn
+		echo "You decided to do it, here we go..."
+		dmesg -c >/dev/null
+		$INSMOD "./$MODULE_NAME"
 	elif [ "$QCINSTCMD" = "unload" ]; then
 		echo "Trying to unload QuickCam driver..."
 		$RMMOD quickcam || $RMMOD mod_quickcam
@@ -143,9 +168,44 @@ echo "see this prompt: --->. Sometimes you're asked root password."
 echo "Pay special attention to lines beginning with [!]."
 echo "It means that some trouble has been detected."
 echo ""
+echo "To most important location is the path to your kernel source"
+echo "or headers. This can be guessed, but you can specify it by"
+echo "giving it as an argument to this script like this:"
+echo "	./quickcam.sh LINUX_DIR=/usr/src/linux"
+echo ""
+#echo "Other possible parameters, for which the default values should"
+#echo "be good, are MODULE_DIR= for kernel module directory,"
+#echo "PREFIX= for utility program directory, USER_OPT= for special"
+#echo "driver parameters, and CC= for C compiler."
+#echo ""
 echo "If you haven't done it yet, now it would be a good moment to"
 echo "take a look at file README."
 # }}}
+# {{{ [fold] Evaluate parameters
+while [ "$1" != "" ]; do
+	echo "Argument found: $1"
+	eval "$1"
+	shift
+done
+if [ "$LINUX_DIR" != "" ]; then
+	export LINUX_DIR
+fi
+if [ "$MODULE_DIR" != "" ]; then
+	export MODULE_DIR
+fi
+if [ "$PREFIX" != "" ]; then
+	export PREFIX
+fi
+if [ "$USER_OPT" != "" ]; then
+	export USER_OPT
+fi
+if [ "$CC" != "" ]; then
+	export CC
+fi
+if [ "$LD" != "" ]; then
+	export LD
+fi
+# }}}
 # {{{ [fold] Test programs and versions
 echo ""
 echo "Next I'm going to check if you have some important programs installed"
@@ -273,30 +333,111 @@ echo "Previous round done. Now checking if you have kernel source installed."
 askreturn
 KERNEL_SOURCE="`make | grep LINUX_DIR | tail -n 1 | awk -F : '{print $2}' | awk '{print $1}'`"
 echo "Kernel source directory: $KERNEL_SOURCE"
-if [ ! -r "$KERNEL_SOURCE/include/linux/version.h" ]; then
+if [ ! -r "$KERNEL_SOURCE/include/linux/videodev.h" ]; then
 	echo "[!] Can not find kernel source or even headers."
 	echo "Make sure that they are installed (install with e.g. rpm or apt-get"
 	echo "if necessary) and ensure that you have read rights to the files."
 	askreturnfail
 fi
+HAVEFULLSRC="y"
 if [ ! -r "$KERNEL_SOURCE/Makefile" ]; then
+	HAVEFULLSRC="n"
 	echo "[!] Kernel headers installed, but not complete source code."
 	echo "Installation still may work with some architectures like Intel x86,"
 	echo "but will definitely fail on others. Note that for kernel 2.6.x"
 	echo "we need always full source code."
 	askreturn
+else
+	KERN_PATCHLEVEL=`awk -F = '/^PATCHLEVEL *=/ {print $2}' < "$KERNEL_SOURCE/Makefile"|tr -d ' 	'`
+	KERN_VERSION=`awk -F = '/^VERSION *=/ {print $2}' < "$KERNEL_SOURCE/Makefile"|tr -d ' 	'`
+	echo "Detected kernel version is $KERN_VERSION.$KERN_PATCHLEVEL.x."
+	if [ "$KERN_VERSION" != "2" ]; then
+		echo "[!] Kernel major version is not 2, which is completely something"
+		echo "not expected. Either it was misdetected or you are using"
+		echo "not supported kernel version."
+		askreturnfail
+	fi
+fi
+if [ ! -r "$KERNEL_SOURCE/.config" ]; then
+	HAVEFULLSRC="n"
+	echo "[!] Kernel configuration file $KERNEL_SOURCE/.config not found."
+	echo "If the headers have been already configured properly, you might"
+	echo "not need it. But it would be better not to trust this, you"
+	echo "really should know where is your configuration file, and"
+	echo "copy it into its place. Often it can be found in /boot/"
+	echo "directory with a name like /boot/config-2.4.26 or something."
+	askreturnfail
+fi
+
+KERNELOK="y"
+if [ ! -r "$KERNEL_SOURCE/include/linux/version.h" ]; then
+	echo "[!] Can not find version.h in kernel source."
+	KERNELOK="n"
 fi
 if [ ! -r "$KERNEL_SOURCE/include/linux/autoconf.h" ]; then
 	echo "[!] Kernel source is not configured properly."
-	echo "You must copy configuration file for the currently running"
-	echo "kernel into file $KERNEL_SOURCE/.config (if it's not there already)"
-	echo "and the give command"
-	echo "	cd $KERNEL_SOURCE && make oldconfig"
-	askreturnfail
+	KERNELOK="n"
 fi
+if [ "$KERNELOK" != "y" ]; then
+	if [ "$HAVEFULLSRC" == "y" ]; then
+		echo "You have full kernel source code but it is not configured"
+		echo "properly. You can configure it by entering the source"
+		echo "directory and typing (with 2.2.x and 2.4.x kernel versions)"
+		echo "	make oldconfig"
+		echo "	make dep"
+		echo "or (with 2.6.x kernel versions)"
+		echo "	make oldconfig"
+		echo "	make modules_prepare"
+		echo "It is also good idea to first clean up completely the kernel"
+		echo "source by typing \"make mrproper\", but this will also delete the"
+		echo ".config file, which has to be copied somewhere else to keep safe."
+		echo "I can also try to do this automatically, in which case"
+		echo "just keep pressing enter, otherwise abort now."
+		askreturn
+		echo "You want me to configure the kernel automatically. All right."
+		echo "If the kernel configuration file doesn't match exactly the"
+		echo "kernel source, you will be asked some questions."
+		echo "This is not a good sign, but if the difference is very small"
+		echo "(maybe a few unimportant questions) it still might work."
+		echo "Usually you can just select the default answer for the questions."
+		askreturn
+		if [ ! -w "$KERNEL_SOURCE" ]; then
+			echo "You don't have write permission to the kernel source,"
+			echo "so I must obtain root access to configure it. Type"
+			echo "the root password now (Ctrl+D to cancel):"
+			QCINSTCMD="conf" KERN_PATCHLEVEL="$KERN_PATCHLEVEL" LINUX_DIR="$LINUX_DIR" su -p -c "$0"
+		else
+			configurekernel
+		fi
+		KERNELOK="y"
+		if [ ! -r "$KERNEL_SOURCE/include/linux/version.h" ]; then
+			echo "[!] Can still not find version.h in kernel source."
+			KERNELOK="n"
+		fi
+		if [ ! -r "$KERNEL_SOURCE/include/linux/autoconf.h" ]; then
+			echo "[!] Kernel source is still not configured properly."
+			KERNELOK="n"
+		fi
+		if [ "$KERNELOK" != "y" ]; then
+			echo "[!] Kernel configuration failed."
+			echo "Check if you got any special messages above."
+			askreturnfail
+		fi
+	else
+		echo "You have only kernel headers but they are not configured"
+		echo "properly. It's pointless trying to continue, this won't work."
+		echo "Either install properly configured kernel headers or full"
+		echo "source with kernel configuration file. In the latter case"
+		echo "I can configure the kernel source using the configuration"
+		echo "file automatically."
+		askreturnfail
+	fi
+fi
+
 KERNEL_VERSION="`make | grep 'Kernel version code' | tail -n 1 | awk -F : '{print $2}' | awk '{print $1}'`"
 KERNEL_UTS=`awk -F \" '/[ 	]*\#[ 	]*define[ 	]*UTS_RELEASE[ 	]*/ { print $2 }' "$KERNEL_SOURCE/include/linux/version.h"|tail -n 1`
 MODULE_NAME="`make | grep 'Driver file name' | tail -n 1 | awk -F : '{print $2}' | awk '{print $1}'`"
+INSTALL_DIR="`make | grep 'Module install directory' | tail -n 1 | awk -F : '{print $2}' | awk '{print $1}'`"
 export MODULE_NAME
 UTS_COUNT=`grep UTS_RELEASE < "$KERNEL_SOURCE/include/linux/version.h" | wc -l`
 if [ $? != 0 ]; then UTS_COUNT=0; fi
@@ -311,12 +452,19 @@ fi
 echo "Kernel version name: $KERNEL_UTS"
 echo "Kernel source version code: $KERNEL_VERSION"
 echo "Driver file name: $MODULE_NAME"
-if [ "$KERNEL_VERSION" -ge "132608" -a ! -w "$KERNEL_SOURCE/Makefile" ]; then
-	echo "[!] You have 2.6.x version kernel but not write permissions"
-	echo "to the source code. For 2.6.x the driver installation"
-	echo "requires write access to the kernel sources."
+echo "Module install directory: $INSTALL_DIR"
+if [ "$KERNEL_VERSION" -ge "$KERNEL26X" -a "$HAVEFULLSRC" != "y" ]; then
+	echo "[!] Not complete kernel 2.6.x source found (possibly just headers)"
+	echo "Remember that pure headers for 2.6.x kernel are not sufficient."
 	askreturnfail
 fi
+# Hmm, it appears that we don't actually need write permission even if not using O= option
+#if [ "$KERNEL_VERSION" -ge "$KERNEL26X" -a ! -w "$KERNEL_SOURCE/Makefile" ]; then
+#	echo "[!] You have 2.6.x version kernel but not write permissions"
+#	echo "to the source code. For 2.6.x the driver installation"
+#	echo "requires write access to the kernel sources."
+#	askreturnfail
+#fi
 if [ $UTS_COUNT = 1 -a "$KERNEL_UTS" != "`uname -r`" ]; then
 	echo "[!] Kernel source version mismatch."
 	echo "This script assumes that the running kernel should be same as"
@@ -340,7 +488,7 @@ echo $RMMODVER | grep module-init-tools >/dev/null
 RMMOD_MIT=$?
 echo $MODPROBEVER | grep module-init-tools >/dev/null
 MODPROBE_MIT=$?
-if [ "$KERNEL_VERSION" -ge 132608 ]; then
+if [ "$KERNEL_VERSION" -ge "$KERNEL26X" ]; then
 	if [ $DEPMOD_MIT != 0 -o $INSMOD_MIT != 0 -o $RMMOD_MIT != 0 -o $MODPROBE_MIT != 0 ]; then
 		echo "[!] Using modutils with 2.6.x kernel."
 		echo "2.6.x requires newer module-init-tools."
@@ -439,7 +587,7 @@ cat /proc/modules | awk '{print $1}' | egrep '(^quickcam|^mod_quickcam)' >/dev/n
 if [ $? = 0 ]; then
 	echo "[!] The QuickCam driver is already loaded!"
 	echo "You should first remove the (old?) module by issuing"
-	echo "	rmmod mod_quickcam || quickcam"
+	echo "	rmmod mod_quickcam || rmmod quickcam"
 	echo "as root, otherwise I will fail to install the new module."
 	echo "I will now try to unload it for you automatically,"
 	echo "if you just give me the root password (Ctrl+D to cancel):"
@@ -543,7 +691,11 @@ VIDEODEV_REAL="`$REALPATH $VIDEODEV`"
 echo "I will be using $VIDEODEV, if there are more cameras I'll not test them."
 askreturn
 echo "Testing if $VIDEODEV is correct."
-ls -la "$VIDEODEV" "$VIDEODEV_REAL"
+ls -la "$VIDEODEV"
+if [ "$VIDEODEV" != "$VIDEODEV_REAL" ]; then
+	ls -la "$VIDEODEV_REAL"
+	echo "$VIDEODEV is a symbolic link to $VIDEODEV_REAL."
+fi
 if [ ! -r $VIDEODEV_REAL -o ! -w $VIDEODEV_REAL ]; then
 	echo "[!] You don't have read/write access to $VIDEODEV."
 	echo "On many distributions, you should add yourself into the"
@@ -565,13 +717,23 @@ CAMERA_MIN=`echo $VIDEODEV | sed 's,.*/[^0-9]*\([0-9]*\),\1,g'`
 if [ "$CAMERA_MIN" != "$VIDEODEV_MIN" ]; then
 	echo "[!] Bad minor number $VIDEODEV_MIN in $VIDEODEV_REAL, should be $CAMERA_MIN."
 	echo "To correct this problem, remove the bad file $VIDEODEV_REAL and"
-	echo "recreate it with mknod-command (read man mknod)."
+	echo "recreate it with mknod-command (read man mknod). Example:"
+	echo "	rm -f $VIDEODEV"
+	echo "	mknod $VIDEODEV c 81 $CAMERA_MIN"
+	echo "	chmod a+rw $VIDEODEV"
 	askreturnfail
 fi
 # }}}
 # {{{ [fold] Final test
 echo ""
-echo "Right now driver is loaded and ready to run."
+echo "Right now driver is loaded and should be ready to run."
+echo "Let's test if user applications can see it, starting with qcset."
+askreturn
+./qcset "$VIDEODEV" -i | head -n 1 | grep 'Logitech QuickCam USB'
+if [ $? != 0 ]; then
+	echo "[!] qcset did not found the QuickCam camera"
+	askreturnfail
+fi
 echo "If you like, you can quit now and start using the camera -"
 echo "you have good chances that it works, if no problems were detected."
 echo "If you have X Window System running and xawtv installed,"
@@ -585,6 +747,8 @@ if [ $? != 0 ]; then
 	askreturnfail
 fi
 echo "Launching xawtv (press q on xawtv window to quit it)..."
+echo "If the image is not sharp, try focusing it by turning the"
+echo "wheel around the camera lens."
 echo "	xawtv -noscale -noxv -c \"$VIDEODEV\""
 xawtv -noscale -noxv -c "$VIDEODEV"
 # }}}
@@ -599,6 +763,10 @@ echo "the utility (qcset) will be now copied into system"
 echo "directories. If you have already other versions,"
 echo "they will be overwritten. Verify by giving root password."
 QCINSTCMD="inst" su -p -c "$0"
+if [ ! -f "$INSTALL_DIR/misc/$MODULE_NAME" ]; then
+	echo "[!] Module install failed to $INSTALL_DIR/misc/$MODULE_NAME"
+	askreturnfail
+fi
 echo "Hopefully the driver is now installed and can be loaded"
 echo "with command"
 echo "	modprobe quickcam"
