@@ -1,15 +1,17 @@
 /*****************************************************************************
  *
- *  quickcam.c - Logitech Quickcam Express Viewer 
+ *  show.c - Logitech QuickCam Express Viewer / Debug utility
  *
  *  Copyright (C) 2000 Georg Acher (acher@in.tum.de)
  *  Copyright (C) 2000 Mark Cave-Ayland (mca198@ecs.soton.ac.uk)
  *  Copyright (C) 2002 Tuukka Toivonen (tuukkat@ee.oulu.fi)
  *
- *   Needs mod_quickcam-module (with V4L interface) and USB support!
+ *  Requires quickcam-module (with V4L interface) and USB support!
  *
  *****************************************************************************/
 
+/* Note: this program is meant only for debugging. It contains very ugly code */
+
 /* Streamformat, as delivered by the camera driver:
    Raw image data for Bayer-RGB-matrix:
    G R for even rows
@@ -32,6 +34,9 @@
 #include <sys/ioctl.h>
 #include <errno.h>
 #include <inttypes.h>
+#include <unistd.h>
+
+#define DUMPDATA 0		/* If using this, set DUMPDATA also in quickcam.h */
 
 #define NOKERNEL
 #define COMPRESS 1
@@ -66,7 +71,7 @@ struct qc_mjpeg_data {
 	void *table_bU[256];
 	void (*yuv2rgb_func)(struct qc_mjpeg_data *, u8 *, u8 *, u8 *, u8 *, void *, void *, int);
 };
-int debug = 0;
+int qcdebug = 0;
 #define IDEBUG_INIT(x)
 #define IDEBUG_EXIT(x)
 #define IDEBUG_TEST(x)
@@ -228,7 +233,7 @@ static void bayer_equalize(unsigned char *bay, int bay_line,
 }
 
 /* Apply the look-up table to the Bayer image data */
-/* 391721-437413 */
+/* 391721-437413 PII, 295471 PIII */
 static void bayer_lut(unsigned char *bay, int bay_line, 
 	unsigned int columns, unsigned int rows, unsigned char *lut)
 {
@@ -322,7 +327,6 @@ static inline void bayer_to_rgb_noip(unsigned char *bay, int bay_line,
 }		  
 
 /* Interpolate R,G,B horizontally. columns must be even */
-/* 2735776-3095322 clocks */
 static inline void bayer_to_rgb_horip(unsigned char *bay, int bay_line,
 		  unsigned char *rgb, int rgb_line,
 		  unsigned int columns, unsigned int rows, int bpp) 
@@ -365,7 +369,6 @@ static inline void bayer_to_rgb_horip(unsigned char *bay, int bay_line,
 }		  
 
 /* Interpolate R,G,B fully. columns and rows must be even */
-/* 2714077-2827455 clocks */
 static inline void bayer_to_rgb_ip(unsigned char *bay, int bay_line,
 		  unsigned char *rgb, int rgb_line,
 		  unsigned int columns, unsigned int rows, int bpp) 
@@ -455,7 +458,6 @@ static inline void bayer_to_rgb_ip(unsigned char *bay, int bay_line,
 }		  
 
 
-/* 2167685 clock cycles */
 static inline void bayer_to_rgb_cott(unsigned char *bay, int bay_line,
 		   unsigned char *rgb, int rgb_line,
 		   int columns, int rows, int bpp)
@@ -507,7 +509,6 @@ static inline void bayer_to_rgb_cott(unsigned char *bay, int bay_line,
 	write_rgb(cur_rgb+rgb_line+bpp, bpp, cur_bay[1], cur_bay[bay_line+1], cur_bay[bay_line]);
 }		  
 
-/* 2133302 clock cycles */
 static inline void bayer_to_rgb_cottnoip(unsigned char *bay, int bay_line,
 		   unsigned char *rgb, int rgb_line,
 		   int columns, int rows, int bpp)
@@ -557,9 +558,340 @@ static inline void bayer_to_rgb_cottnoip(unsigned char *bay, int bay_line,
 	write_rgb(cur_rgb+bpp,          bpp, cur_bay[1], cur_bay[bay_line+1], cur_bay[bay_line]);
 	write_rgb(cur_rgb+rgb_line,     bpp, cur_bay[1], cur_bay[bay_line+1], cur_bay[bay_line]);
 	write_rgb(cur_rgb+rgb_line+bpp, bpp, cur_bay[1], cur_bay[bay_line+1], cur_bay[bay_line]);
+}
+
+
+#if 0
+	/* Bilinear filtering */
+	static int wrg0 = 0;	/* Weight for Red on Green */
+	static int wbg0 = 0;
+	static int wgr0 = 0;
+	static int wbr0 = 0;
+	static int wgb0 = 0;
+	static int wrb0 = 0;
+#elif 1
+	/* Best PSNR */
+	static int wrg0 = 144;
+	static int wbg0 = 160;
+	static int wgr0 = 120;
+	static int wbr0 = 192;
+	static int wgb0 = 120;
+	static int wrb0 = 168;
+#elif 0
+	/* Very sharp */
+	static int wrg0 = ;
+	static int wbg0 = ;
+	static int wgr0 = ;
+	static int wbr0 = ;
+	static int wgb0 = ;
+	static int wrb0 = ;
+#else
+	/* Sharp with round multipliers */
+	static int wrg0 = 256;
+	static int wbg0 = 256;
+	static int wgr0 = 128;
+	static int wbr0 = 128;
+	static int wgb0 = 256;
+	static int wrb0 = 256;
+#endif
+	static unsigned int sharpness = 32767;
+
+	int wrg;
+	int wbg;
+	int wgr;
+	int wbr;
+	int wgb;
+	int wrb;
+
+
+static void bayer_to_rgb_gptm_printval(void) {
+	printf("0: wrg %i wbg %i wgr %i wbr %i wgb %i wrb %i\n", wrg0, wbg0, wgr0, wbr0, wgb0, wrb0);
+	printf("f: wrg %i wbg %i wgr %i wbr %i wgb %i wrb %i\n", wrg, wbg, wgr, wbr, wgb, wrb);
+	printf("sharpness = %i\n", sharpness);
+}
+
+static inline void bayer_to_rgb_gptm(unsigned char *bay, int bay_line,
+		   unsigned char *rgb, int rgb_line,
+		   int columns, int rows, int bpp)
+{
+
+	/* 0.4 fixed point weights, should be between 0-16. Larger value = sharper */
+	unsigned int wu;
+	int r,g,b,w;
+	unsigned char *cur_bay, *cur_rgb;
+	int bay_line2, bay_line3, rgb_line2;
+	int total_columns;
+
+	/* Compute weights */
+	wu = (sharpness * sharpness) >> 16;
+ 	wu = (wu * wu) >> 16;
+	wrg = (wrg0 * wu) >> 10;
+	wbg = (wbg0 * wu) >> 10;
+	wgr = (wgr0 * wu) >> 10;
+	wbr = (wbr0 * wu) >> 10;
+	wgb = (wgb0 * wu) >> 10;
+	wrb = (wrb0 * wu) >> 10;
+//	bayer_to_rgb_gptm_printval();
+
+	/* Process 2 lines and rows per each iteration, but process the first and last two columns and rows separately */
+	total_columns = (columns>>1) - 2;
+	rows = (rows>>1) - 2;
+	bay_line2 = 2*bay_line;
+	bay_line3 = 3*bay_line;
+	rgb_line2 = 2*rgb_line;
+
+	/* Process first two pixel rows here */
+	cur_bay = bay;
+	cur_rgb = rgb;
+	columns = total_columns + 2;
+	do {
+		write_rgb(cur_rgb+0,            bpp, cur_bay[1], cur_bay[0],          cur_bay[bay_line]);
+		write_rgb(cur_rgb+bpp,          bpp, cur_bay[1], cur_bay[0],          cur_bay[bay_line]);
+		write_rgb(cur_rgb+rgb_line,     bpp, cur_bay[1], cur_bay[bay_line+1], cur_bay[bay_line]);
+		write_rgb(cur_rgb+rgb_line+bpp, bpp, cur_bay[1], cur_bay[bay_line+1], cur_bay[bay_line]);
+		cur_bay += 2;
+		cur_rgb += 2*bpp;
+	} while (--columns);
+	bay += bay_line2;
+	rgb += rgb_line2;
+
+	do {
+		cur_bay = bay;
+		cur_rgb = rgb;
+		columns = total_columns;
+		
+		/* Process first 2x2 pixel block in a row here */
+		write_rgb(cur_rgb+0,            bpp, cur_bay[1], cur_bay[0],          cur_bay[bay_line]);
+		write_rgb(cur_rgb+bpp,          bpp, cur_bay[1], cur_bay[0],          cur_bay[bay_line]);
+		write_rgb(cur_rgb+rgb_line,     bpp, cur_bay[1], cur_bay[bay_line+1], cur_bay[bay_line]);
+		write_rgb(cur_rgb+rgb_line+bpp, bpp, cur_bay[1], cur_bay[bay_line+1], cur_bay[bay_line]);
+		cur_bay += 2;
+		cur_rgb += 2*bpp;
+
+		do {
+			w = 4*cur_bay[0] - (cur_bay[-bay_line-1] + cur_bay[-bay_line+1] + cur_bay[bay_line-1] + cur_bay[bay_line+1]);
+			r = (512*(cur_bay[-1] + cur_bay[1]) + w*wrg) >> 10;
+			b = (512*(cur_bay[-bay_line] + cur_bay[bay_line]) + w*wbg) >> 10;
+			write_rgb(cur_rgb+0, bpp, CLIP(r,0,255), cur_bay[0], CLIP(b,0,255));
+
+			w = 4*cur_bay[1] - (cur_bay[-bay_line2+1] + cur_bay[-1] + cur_bay[3] + cur_bay[bay_line2+1]);
+			g = (256*(cur_bay[-bay_line+1] + cur_bay[0] + cur_bay[2] + cur_bay[bay_line+1]) + w*wgr) >> 10;
+			b = (256*(cur_bay[-bay_line] + cur_bay[-bay_line+2] + cur_bay[bay_line] + cur_bay[bay_line+2]) + w*wbr) >> 10;
+			write_rgb(cur_rgb+bpp, bpp, cur_bay[1], CLIP(g,0,255), CLIP(b,0,255));
+
+			w = 4*cur_bay[bay_line] - (cur_bay[-bay_line] + cur_bay[bay_line-2] + cur_bay[bay_line+2] + cur_bay[bay_line3]);
+			r = (256*(cur_bay[-1] + cur_bay[1] + cur_bay[bay_line2-1] + cur_bay[bay_line2+1]) + w*wrb) >> 10;
+			g = (256*(cur_bay[0] + cur_bay[bay_line-1] + cur_bay[bay_line+1] + cur_bay[bay_line2]) + w*wgb) >> 10;
+			write_rgb(cur_rgb+rgb_line, bpp, CLIP(r,0,255), CLIP(g,0,255), cur_bay[bay_line]);
+
+			w = 4*cur_bay[bay_line+1] - (cur_bay[0] + cur_bay[2] + cur_bay[bay_line2] + cur_bay[bay_line2+2]);
+			r = (512*(cur_bay[1] + cur_bay[bay_line2+1]) + w*wrg) >> 10;
+			b = (512*(cur_bay[bay_line] + cur_bay[bay_line+2]) + w*wbg) >> 10;
+			write_rgb(cur_rgb+rgb_line+bpp, bpp, CLIP(r,0,255), cur_bay[bay_line+1], CLIP(b,0,255));
+
+			cur_bay += 2;
+			cur_rgb += 2*bpp;
+		} while (--columns);
+
+		/* Process last 2x2 pixel block in a row here */
+		write_rgb(cur_rgb+0,            bpp, cur_bay[1], cur_bay[0],          cur_bay[bay_line]);
+		write_rgb(cur_rgb+bpp,          bpp, cur_bay[1], cur_bay[0],          cur_bay[bay_line]);
+		write_rgb(cur_rgb+rgb_line,     bpp, cur_bay[1], cur_bay[bay_line+1], cur_bay[bay_line]);
+		write_rgb(cur_rgb+rgb_line+bpp, bpp, cur_bay[1], cur_bay[bay_line+1], cur_bay[bay_line]);
+
+		bay += bay_line2;
+		rgb += rgb_line2;
+	} while (--rows);
+
+	/* Process last two pixel rows here */
+	cur_bay = bay;
+	cur_rgb = rgb;
+	columns = total_columns + 2;
+	do {
+		write_rgb(cur_rgb+0,            bpp, cur_bay[1], cur_bay[0],          cur_bay[bay_line]);
+		write_rgb(cur_rgb+bpp,          bpp, cur_bay[1], cur_bay[0],          cur_bay[bay_line]);
+		write_rgb(cur_rgb+rgb_line,     bpp, cur_bay[1], cur_bay[bay_line+1], cur_bay[bay_line]);
+		write_rgb(cur_rgb+rgb_line+bpp, bpp, cur_bay[1], cur_bay[bay_line+1], cur_bay[bay_line]);
+		cur_bay += 2;
+		cur_rgb += 2*bpp;
+	} while (--columns);
+}
+
+/* faster version of bayer_to_rgb_gptm with fixed multipliers */
+static inline void bayer_to_rgb_gptm_fast(unsigned char *bay, int bay_line,
+		   unsigned char *rgb, int rgb_line,
+		   int columns, int rows, int bpp)
+{
+	int r,g,b,w;
+	unsigned char *cur_bay, *cur_rgb;
+	int bay_line2, bay_line3, rgb_line2;
+	int total_columns;
+
+	/* Process 2 lines and rows per each iteration, but process the first and last two columns and rows separately */
+	total_columns = (columns>>1) - 2;
+	rows = (rows>>1) - 2;
+	bay_line2 = 2*bay_line;
+	bay_line3 = 3*bay_line;
+	rgb_line2 = 2*rgb_line;
+
+	/* Process first two pixel rows here */
+	cur_bay = bay;
+	cur_rgb = rgb;
+	columns = total_columns + 2;
+	do {
+		write_rgb(cur_rgb+0,            bpp, cur_bay[1], cur_bay[0],          cur_bay[bay_line]);
+		write_rgb(cur_rgb+bpp,          bpp, cur_bay[1], cur_bay[0],          cur_bay[bay_line]);
+		write_rgb(cur_rgb+rgb_line,     bpp, cur_bay[1], cur_bay[bay_line+1], cur_bay[bay_line]);
+		write_rgb(cur_rgb+rgb_line+bpp, bpp, cur_bay[1], cur_bay[bay_line+1], cur_bay[bay_line]);
+		cur_bay += 2;
+		cur_rgb += 2*bpp;
+	} while (--columns);
+	bay += bay_line2;
+	rgb += rgb_line2;
+
+	do {
+		cur_bay = bay;
+		cur_rgb = rgb;
+		columns = total_columns;
+		
+		/* Process first 2x2 pixel block in a row here */
+		write_rgb(cur_rgb+0,            bpp, cur_bay[1], cur_bay[0],          cur_bay[bay_line]);
+		write_rgb(cur_rgb+bpp,          bpp, cur_bay[1], cur_bay[0],          cur_bay[bay_line]);
+		write_rgb(cur_rgb+rgb_line,     bpp, cur_bay[1], cur_bay[bay_line+1], cur_bay[bay_line]);
+		write_rgb(cur_rgb+rgb_line+bpp, bpp, cur_bay[1], cur_bay[bay_line+1], cur_bay[bay_line]);
+		cur_bay += 2;
+		cur_rgb += 2*bpp;
+
+		do {
+			w = 4*cur_bay[0] - (cur_bay[-bay_line-1] + cur_bay[-bay_line+1] + cur_bay[bay_line-1] + cur_bay[bay_line+1]);
+			r = (2*(cur_bay[-1] + cur_bay[1]) + w) >> 2;
+			b = (2*(cur_bay[-bay_line] + cur_bay[bay_line]) + w) >> 2;
+			write_rgb(cur_rgb+0, bpp, CLIP(r,0,255), cur_bay[0], CLIP(b,0,255));
+
+			w = 4*cur_bay[1] - (cur_bay[-bay_line2+1] + cur_bay[-1] + cur_bay[3] + cur_bay[bay_line2+1]);
+			g = (2*(cur_bay[-bay_line+1] + cur_bay[0] + cur_bay[2] + cur_bay[bay_line+1]) + w) >> 3;
+			b = (2*(cur_bay[-bay_line] + cur_bay[-bay_line+2] + cur_bay[bay_line] + cur_bay[bay_line+2]) + w) >> 3;
+			write_rgb(cur_rgb+bpp, bpp, cur_bay[1], CLIP(g,0,255), CLIP(b,0,255));
+
+			w = 4*cur_bay[bay_line] - (cur_bay[-bay_line] + cur_bay[bay_line-2] + cur_bay[bay_line+2] + cur_bay[bay_line3]);
+			r = ((cur_bay[-1] + cur_bay[1] + cur_bay[bay_line2-1] + cur_bay[bay_line2+1]) + w) >> 2;
+			g = ((cur_bay[0] + cur_bay[bay_line-1] + cur_bay[bay_line+1] + cur_bay[bay_line2]) + w) >> 2;
+			write_rgb(cur_rgb+rgb_line, bpp, CLIP(r,0,255), CLIP(g,0,255), cur_bay[bay_line]);
+
+			w = 4*cur_bay[bay_line+1] - (cur_bay[0] + cur_bay[2] + cur_bay[bay_line2] + cur_bay[bay_line2+2]);
+			r = (2*(cur_bay[1] + cur_bay[bay_line2+1]) + w) >> 2;
+			b = (2*(cur_bay[bay_line] + cur_bay[bay_line+2]) + w) >> 2;
+			write_rgb(cur_rgb+rgb_line+bpp, bpp, CLIP(r,0,255), cur_bay[bay_line+1], CLIP(b,0,255));
+
+			cur_bay += 2;
+			cur_rgb += 2*bpp;
+		} while (--columns);
+
+		/* Process last 2x2 pixel block in a row here */
+		write_rgb(cur_rgb+0,            bpp, cur_bay[1], cur_bay[0],          cur_bay[bay_line]);
+		write_rgb(cur_rgb+bpp,          bpp, cur_bay[1], cur_bay[0],          cur_bay[bay_line]);
+		write_rgb(cur_rgb+rgb_line,     bpp, cur_bay[1], cur_bay[bay_line+1], cur_bay[bay_line]);
+		write_rgb(cur_rgb+rgb_line+bpp, bpp, cur_bay[1], cur_bay[bay_line+1], cur_bay[bay_line]);
+
+		bay += bay_line2;
+		rgb += rgb_line2;
+	} while (--rows);
+
+	/* Process last two pixel rows here */
+	cur_bay = bay;
+	cur_rgb = rgb;
+	columns = total_columns + 2;
+	do {
+		write_rgb(cur_rgb+0,            bpp, cur_bay[1], cur_bay[0],          cur_bay[bay_line]);
+		write_rgb(cur_rgb+bpp,          bpp, cur_bay[1], cur_bay[0],          cur_bay[bay_line]);
+		write_rgb(cur_rgb+rgb_line,     bpp, cur_bay[1], cur_bay[bay_line+1], cur_bay[bay_line]);
+		write_rgb(cur_rgb+rgb_line+bpp, bpp, cur_bay[1], cur_bay[bay_line+1], cur_bay[bay_line]);
+		cur_bay += 2;
+		cur_rgb += 2*bpp;
+	} while (--columns);
+}
+
+/* Copy only those values into RGB image, which are in the Bayer image too, leave the rest zero */
+static inline void bayer_to_rgb_copyonly(unsigned char *bay, int bay_line,
+		   unsigned char *rgb, int rgb_line,
+		   int columns, int rows, int bpp)
+{
+	unsigned char *cur_bay, *cur_rgb;
+	int bay_line2, rgb_line2;
+	int total_columns;
+
+	/* Process 2 lines and rows per each iteration */
+	total_columns = columns >> 1;
+	rows >>= 1;
+	bay_line2 = 2*bay_line;
+	rgb_line2 = 2*rgb_line;
+	do {
+		cur_bay = bay;
+		cur_rgb = rgb;
+		columns = total_columns;
+		do {
+			write_rgb(cur_rgb+0,            bpp, 0,          cur_bay[0],          0);
+			write_rgb(cur_rgb+bpp,          bpp, cur_bay[1], 0,                   0);
+			write_rgb(cur_rgb+rgb_line,     bpp, 0,          0,                   cur_bay[bay_line]);
+			write_rgb(cur_rgb+rgb_line+bpp, bpp, 0,          cur_bay[bay_line+1], 0);
+			cur_bay += 2;
+			cur_rgb += 2*bpp;
+		} while (--columns);
+		bay += bay_line2;
+		rgb += rgb_line2;
+	} while (--rows);
 }		  
 
 
+void bayer_to_rgb(unsigned char *bay, int bay_line, unsigned char *rgb, int rgb_line, int columns, int rows)
+{
+	/* Measure 100 frames at highest optimization level, cycles per pixel */
+//	bayer_to_rgb_noip(bay, bay_line, rgb, rgb_line, columns, rows, 4);		/* 20.29 */
+//	bayer_to_rgb_horip(bay, bay_line, rgb, rgb_line, columns, rows, 4);		/* 21.77 */
+//	bayer_to_rgb_ip(bay, bay_line, rgb, rgb_line, columns, rows, 4);		/* 21.02 */
+//	bayer_to_rgb_cott(bay, bay_line, rgb, rgb_line, columns, rows, 4);		/* 21.25 */
+//	bayer_to_rgb_cottnoip(bay, bay_line, rgb, rgb_line, columns, rows, 4);		/* 21.35 */
+//	bayer_to_rgb_gptm(bay, bay_line, rgb, rgb_line, columns, rows, 4);		/* 42.85 (36.62 with unsigned multipliers) */
+//	bayer_to_rgb_gptm_fast(bay, bay_line, rgb, rgb_line, columns, rows, 4);		/* 37.44 */
+	bayer_to_rgb_copyonly(bay, bay_line, rgb, rgb_line, columns, rows, 4);		/* 20.48 */
+}
+
+static inline void grey_to_buffer(unsigned char *grey, int grey_line, unsigned char *rgb, int rgb_line, int columns, int rows, int bpp)
+{
+	unsigned char *cur_grey, *cur_rgb;
+	int total_columns = columns;
+	do {
+		cur_grey = grey;
+		cur_rgb  = rgb;
+		columns  = total_columns;
+		do {
+			write_rgb(cur_rgb, bpp, *cur_grey, *cur_grey, *cur_grey);
+			cur_grey++;
+			cur_rgb += bpp;
+		} while (--columns);
+		grey += grey_line;
+		rgb  += rgb_line;
+	} while (--rows);
+}
+
+
+/*
+ Copyright (c) 2000 Jeroen B. Vreeken (pe1rxq@amsat.org)
+ from Endpoints (formerly known as AOX) se401 USB Camera Driver
+*/
+/*
+        This shouldn't really be done in a v4l driver....
+        But it does make the image look a lot more usable.
+        Basically it lifts the dark pixels more than the light pixels.
+*/
+/* 1195964 clock cycles on PIII, 4 times slower than LUT-approach */
+static inline void enhance_picture(unsigned char *frame, int len)
+{
+        while (len--) {
+                *frame=(((*frame^255)*(*frame^255))/255)^255;
+                frame++;
+        }
+}
 
 /*-----------------------------------------------------------------*/
 void initializeX (int width, int height)
@@ -601,7 +933,7 @@ void initializeX (int width, int height)
 	else
 		bypp=2;
 
-	XStoreName (dpy, win, "Quickcam");
+	XStoreName (dpy, win, "QuickCam");
 	XMapWindow (dpy, win);
 
 	gc = XCreateGC (dpy, win, 0,  xgc);
@@ -883,14 +1215,16 @@ void print_cap(struct video_capability *vidcap, struct video_window *vidwin,
 int main(int argc, char** argv)
 {
 	struct qc_mjpeg_data mjpeg_data;
+//	int usepalette = VIDEO_PALETTE_GREY;
 	int usepalette = VIDEO_PALETTE_RGB24;
 //	int usepalette = VIDEO_PALETTE_BAYER;
 //	int usepalette = VIDEO_PALETTE_MJPEG;
 	int fd,r,y,x;
 	int W;
 	int H;
+	int flag = 0;
 	int mode=0; // 0:352*288, 1: 176*144
-	char *device = "/dev/video";
+	char *device = "/dev/video0";
 	int argp = 1;
 	unsigned char *buffer;
 	int buffersize;
@@ -904,6 +1238,9 @@ int main(int argc, char** argv)
 	int framenum = 0;
 	time_t begintime=0,nowtime;
 	
+#if DUMPDATA
+	usepalette = VIDEO_PALETTE_GREY;
+#endif
 	if (argp < argc && memcmp("/dev", argv[argp], 4) == 0)
 		device = argv[argp++];
 
@@ -919,7 +1256,7 @@ int main(int argc, char** argv)
 	}
 	if (mode>1)
 	{
-		fprintf(stderr, "Usage: quickcam <mode>\n"
+		fprintf(stderr, "Usage: show <mode>\n"
 			"mode: 0 (352*288), 1 (176*144)\n");
 		exit(0);
 	}
@@ -929,6 +1266,10 @@ int main(int argc, char** argv)
 
 	vidwin.width = vidcap.maxwidth;///2 + 1;
 	vidwin.height = vidcap.maxheight;///2 + 1;
+//vidwin.width=48;
+//vidwin.height=32;
+//vidwin.width=360;
+//vidwin.height=270;
 
 	r=ioctl(fd, VIDIOCSWIN, &vidwin);
 	if (r!=0) { perror("ioctl VIDIOCSWIN"); exit(1); }
@@ -941,7 +1282,7 @@ int main(int argc, char** argv)
 	if (r!=0) { perror("ioctl VIDIOCSCHAN"); exit(1); }
 	r=ioctl(fd, VIDIOCGPICT, &vidpic);
 	if (r!=0) { perror("ioctl VIDIOCGPICT"); exit(1); }
-	vidpic.depth = 24;
+	vidpic.depth = (usepalette!=VIDEO_PALETTE_BAYER && usepalette!=VIDEO_PALETTE_GREY) ? 24 : 8;
 	vidpic.palette = usepalette;
 	r=ioctl(fd, VIDIOCSPICT, &vidpic);
 	if (r!=0) { perror("ioctl VIDIOCSPICT"); exit(1); }
@@ -951,8 +1292,8 @@ int main(int argc, char** argv)
 
 	W = vidwin.width;
 	H = vidwin.height;
-	printf("using frame size: %i x %i\n",W,H);
-	buffersize = usepalette==VIDEO_PALETTE_BAYER ? H*W : 3*H*W;
+	buffersize = (usepalette==VIDEO_PALETTE_BAYER || usepalette==VIDEO_PALETTE_GREY) ? H*W : 3*H*W;
+	printf("using frame size: %i x %i, buffersize=%i\n",W,H,buffersize);
 	buffer = (char *)malloc(buffersize);
 	initializeX(W,H);
 	if (usepalette==VIDEO_PALETTE_MJPEG) {
@@ -963,12 +1304,15 @@ int main(int argc, char** argv)
 	printf("START\n");
 	while (1) {
 		int midvalue, readlen;
-		printf("read()=");
+//		usleep(3000000);
+//		usleep(50000);
+//		printf("read()=");
 		fflush(stdout);
 		readlen = read(fd,buffer,buffersize);
-		printf("%i\n",readlen);
+//		printf("%i\n",readlen);
 		if (readlen!=buffersize && usepalette!=VIDEO_PALETTE_MJPEG) { 
 			printf("Short read by %i, skipping frame\n", buffersize-readlen);
+exit(1);
 			continue; 
 		}
 		if (usepalette==VIDEO_PALETTE_BAYER) {
@@ -978,15 +1322,24 @@ int main(int argc, char** argv)
 //			midvalue = bayer_midvalue(buffer,W, W,H);
 //			printf("midval=%i\n", midvalue);
 //			bayer_equalize(buffer,W, W,H, lut);
-//			bayer_lut(buffer,W, W,H, lut);
 starttime = rdtsc();
-			bayer_to_rgb_cott(buffer,W,shminfo.shmaddr,4*W, W,H, 4);
+//			bayer_lut(buffer,W, W,H, lut);
 stoptime = rdtsc();
+if (flag) enhance_picture(buffer, W*H);
+			bayer_to_rgb(buffer,W,shminfo.shmaddr,4*W, W,H);
 stoptime -= starttime;
 totaltime += stoptime;
 benchmarks++;
 if (stoptime<minimum) minimum=stoptime;
-//printf("Elapsed: %i, Average: %i, Minimum: %i\n", (int)stoptime, (int)(totaltime/benchmarks), (int)minimum);
+printf("Elapsed: %i, Average: %i, Minimum: %i\n", (int)stoptime, (int)(totaltime/benchmarks), (int)minimum);
+		} else if (usepalette==VIDEO_PALETTE_GREY) {
+#if DUMPDATA
+			int w=163; 
+			int h=288;
+			grey_to_buffer(buffer,w, shminfo.shmaddr,4*W, w,h, 4);
+#else
+			grey_to_buffer(buffer,W,shminfo.shmaddr,4*W, W,H, 4);
+#endif
 		} else if (usepalette==VIDEO_PALETTE_MJPEG) {
 			printf("data: %02X %02X %02X %02X %02X   size: %i bytes\n",
 				buffer[0], buffer[1], buffer[2], buffer[3], buffer[4], readlen);
@@ -1006,16 +1359,19 @@ if (stoptime<minimum) minimum=stoptime;
 			framenum++;
 			nowtime = time(NULL);
 			nowtime -= begintime;
-			if (nowtime>1 && framenum%10==0)
+			if (nowtime>1 && framenum%10==0) {
 				printf("%i frames, %i seconds = %i.%i fps\n", framenum, (int)nowtime, (int)(framenum/nowtime), (int)((framenum*10/nowtime) % 10));
+			}
 		}
 		while (XPending(dpy)) {
 			XNextEvent(dpy, &event);
 			if (event.type == KeyPress) {
 				char *c;
 				int step = 1000;
+				int delta = 32;
 				c = XKeysymToString(XKeycodeToKeysym(dpy,event.xkey.keycode,0));
 				r = 0;
+//	printf("wrg %i wbg %i wgr %i wbr %i wgb %i wrb %i\n", wrg, wbg, wgr, wbr, wgb, wrb);
 				if (c[0]!=0 && c[1]==0) switch (c[0]) {
 					case 'q': goto done;
 					case 'a':
@@ -1034,6 +1390,51 @@ if (stoptime<minimum) minimum=stoptime;
 						if (vidpic.contrast <= 0xFFFF-step) vidpic.contrast += step;
 						r = 1;
 						break;
+					case 'r':	wrg0 -= delta;	bayer_to_rgb_gptm_printval();	break;
+					case 't':	wrg0 += delta;	bayer_to_rgb_gptm_printval();	break;
+					case 'f':	wbg0 -= delta;	bayer_to_rgb_gptm_printval();	break;
+					case 'g':	wbg0 += delta;	bayer_to_rgb_gptm_printval();	break;
+					case 'v':	wgr0 -= delta;	bayer_to_rgb_gptm_printval();	break;
+					case 'b':	wgr0 += delta;	bayer_to_rgb_gptm_printval();	break;
+					case 'y':	wbr0 -= delta;	bayer_to_rgb_gptm_printval();	break;
+					case 'u':	wbr0 += delta;	bayer_to_rgb_gptm_printval();	break;
+					case 'h':	wgb0 -= delta;	bayer_to_rgb_gptm_printval();	break;
+					case 'j':	wgb0 += delta;	bayer_to_rgb_gptm_printval();	break;
+					case 'n':	wrb0 -= delta;	bayer_to_rgb_gptm_printval();	break;
+					case 'm':	wrb0 += delta;	bayer_to_rgb_gptm_printval();	break;
+					case 'i':
+						wrg0 -= delta;
+						wbg0 -= delta;
+						wgr0 -= delta;
+						wbr0 -= delta;
+						wgb0 -= delta;
+						wrb0 -= delta;
+						bayer_to_rgb_gptm_printval();	
+						break;
+					case 'o':
+						wrg0 += delta;
+						wbg0 += delta;
+						wgr0 += delta;
+						wbr0 += delta;
+						wgb0 += delta;
+						wrb0 += delta;
+						bayer_to_rgb_gptm_printval();	
+						break;
+					case 'k':
+						sharpness -= 8192/4;
+						printf("sharpness: %i\n", sharpness);
+						break;
+					case 'l':
+						sharpness += 8192/4;
+						printf("sharpness: %i\n", sharpness);
+						break;
+					case 'p':
+						bayer_to_rgb_gptm_printval();
+						printf("sharpness: %i\n", sharpness);
+						break;
+					case 'c':
+						flag = !flag;
+						break;
 					default:
 						printf("Key %s not used (press q to quit)\n", c);
 						break;						
